package flash.accessibility
{
	import flash.display.DisplayObject;

//
// Accessibility
//

/**
 * The Accessibility class manages communication with screen readers. Screen readers are a 
 * type of assistive technology for visually impaired users that provides an audio version of 
 * screen content. The methods of the Accessibility class are static&#x2014;that is, you don't 
 * have to create an instance of the class to use its methods.
 * 
 * <p>To get and set accessible properties for a specific object, such as a button, movie 
 * clip, or text field, use the <code>DisplayObject.accessibilityProperties</code> property. 
 * To determine whether the player is running in an environment that supports accessibility aids, use 
 * the <code>Capabilities.hasAccessibility</code> property. </p>
 *
 * @includeExample examples\AccessibilityExample.as -noswf
 * @see flash.accessibility.AccessibilityProperties
 * @see flash.display.DisplayObject#accessibilityProperties
 * @see flash.system.Capabilities#hasAccessibility
 * @see flash.net.Socket Socket
 * 
 * @keyword Accessibility, Accessibility object, built-in class
 * @playerversion Flash 9
 * @langversion 3.0
 * @review
 */
public final class Accessibility
{
	/**
	* Indicates whether a screen reader is currently active and the player is 
	* communicating with it. Use this method when you want your application to behave 
	* differently in the presence of a screen reader.
	* 
	* <p><strong>Note: </strong>If you call this method within 1 or 2 seconds of the first
	* appearance of the Flash<sup>&#xAE;</sup> window in which your document is playing, you might get a return
	* value of <code>false</code> even if there is an active accessibility 
	* client. This happens because of an asynchronous communication mechanism between Flash 
	* and accessibility clients. You can work around this limitation by ensuring a delay of 1 to 2 
	* seconds after loading your document before calling this method.</p>
	*
	* To determine whether the player is running in an environment that supports screen readers, use the  
	* <code>Capabilities.hasAccessibility</code> property. 
	*
	* @return If Flash Player is communicating with a screen reader, then <code>true</code>; <code>false</code> otherwise.
	*
	* @example The following example checks whether an accessibility aid is currently 
	* active:
	* <listing version="3.0">
	* if (Accessibility.active == true) {
	*   trace ("An accessibility aid is currently active");
	* } else {
	*   trace ("There is currently no active accessibility aid");
	* }
	* </listing>
	*
	* @see flash.system.Capabilities#hasAccessibility
	* @see #updateProperties()
	* @helpid
	* @refpath 
	* @keyword Accessibility.isActive, Accessibility, isActive, screen reader, MSAA
    * @playerversion Flash 9
    * @langversion 3.0
	*/
	public static native function get active():Boolean;
	
	[Inspectable(environment="none")]
	/**
	 * Sends an event to the Microsoft Active Accessibility API. Microsoft Active 
	 * Accessibility handles that event and sends the event to any active screen reader
	 * application, which in turn reports the change to the user.  For example, when a 
	 * user toggles a RadioButton instance, the RadioButton's Accessibility Implementation 
	 * calls Accessibility.sendEvent() with the eventType EVENT_OBJECT_STATECHANGE.
	 * 
	 * @param source 	The DisplayObject from which the accessibility event is being sent.
	 * @param childID 	The child id of the accessibility interface element to which the event applies (for example, an individual list item in a list box). Use 0 to indicate that the event applies to the DisplayObject supplied in the <code>source</code> parameter.
	 * @param eventType	A constant indicating the event type. Event names and values are a subset of the <a href="constants.html#events">MSAA event constants</a>.
	 * @param nonHTML	A Boolean indication of whether or not the event is one of the standard event types that can be generated from an HTML form. When set to <code>true</code>, this parameter helps prevent some problems that may occur with screen readers that interperet Flash content as part of the HTML page.  The default value is <code>false</code>.
	 * 
	 * @see flash.accessibility.AccessibilityImplementation
	 * @see mx.accessibility.AccImpl
	 * @review
	 */
	public static native function sendEvent( source : DisplayObject, childID : uint, eventType : uint, nonHTML : Boolean = false ) : void;

	/**
	* Tells Flash Player to apply any accessibility changes made by using the <code>DisplayObject.accessibilityProperties</code> property.  
	* You need to call this method for your changes to take effect. 
	*
	* <p>If you modify the accessibility properties for multiple objects, only one call to the
	* <code>Accessibility.updateProperties()</code> method is necessary; multiple calls can result in
	* reduced performance and erroneous screen reader output.</p>
	*
	* @throws IllegalOperationError Accessibility is not supported in this version of
	* Flash Player. Do not call the <code>Accessibility.updateProperties()</code> method 
	* if the <code>flash.system.Capabilities.hasAccessibility</code> property is <code>false</code>.
	*
	* @example If you change an image and want to update its accessible description, you 
	* could use the following ActionScript code:
	* <listing version="3.0">if (my_displayObj.accessibilityProperties == undefined ) {
	*   my_displayObj.accessibilityProperties = new AccessibilityProperties();
	* }
	* my_displayObj.accessibilityProperties.name = "Photo of Mount Rushmore";
	* Accessibility.updateProperties();</listing>
	*
	* @see #active 
	* @see flash.accessibility.AccessibilityProperties
	* @see flash.display.DisplayObject#accessibilityProperties
	* @see flash.system.Capabilities#hasAccessibility
	*
	* @helpid 
	* @refpath 
	* @keyword accessibility; Accessibility.updateProperties, updateProperties, screen reader, MSAA
    * @playerversion Flash 9
    * @langversion 3.0
	*/
	public static native function updateProperties():void;
};
	

/*
 * [ggrossman 04/07/05] API SCRUB
 *
 * - Method changed to accessor: _Accessibility.isActive()_ ->
 *   _Accessibility.active_
 *
 * - Marked this class _final_
 */

}
