////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2003-2008 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.accessibility
{

import flash.accessibility.Accessibility;
import flash.accessibility.AccessibilityProperties;
import flash.events.Event;
import flash.events.KeyboardEvent;
import flash.geom.Rectangle;
import flash.ui.Keyboard;

import mx.controls.Menu;
import mx.controls.PopUpButton;
import mx.core.UIComponent;
import mx.core.mx_internal;

use namespace mx_internal;

/**
 *  The PopUpButtonAccImpl class, also called the PopUpButton Accessibility Implementation class, 
 *  enables communication between a PopUpButton component and a screen reader. Screen readers are used 
 *  to translate screen content into synthesized speech or braille for visually impaired users.
 * 
 *  <p>The PopUpButtonAccImpl class supports system roles, object-based events, and states.</p> 
 * 
 *  <p>A PopUpButton reports the role <code>ROLE_SYSTEM_SPLITBUTTON</code> (0x3e) to a screen reader.</p>
 *
 *  <p>A PopUpButton reports the following states to a screen reader:</p>
 *  <ul>
 *      <li><code>STATE_SYSTEM_NORMAL</code> (0x00000000)</li>
 *      <li><code>STATE_SYSTEM_UNAVAILABLE</code> (0x00000001)</li>
 *      <li><code>STATE_SYSTEM_FOCUSED</code> (0x00000004)</li>
 *      <li><code>STATE_SYSTEM_PRESSED</code> (0x00000008)</li>
 *      <li><code>STATE_SYSTEM_FOCUSABLE</code> (0x00100000)</li>
 * 		<li><code>STATE_SYSTEM_HOTTRACKED</code> (0x00000080)</li>
 *  </ul>
 *
 *  <p>A PopUpButton dispatches the following events to a screen reader:</p>
 *  <ul>
 *      <li><code>EVENT_OBJECT_STATECHANGE</code> (0x800A)</li>
 *      <li><code>EVENT_OBJECT_NAMECHANGE</code> (0x800C)</li>
 *  </ul>
 * 
 *  @see mx.controls.PopUpButton
 *  @see mx.accessibility.AccImpl
 *
 *  @helpid 3002
 *  @tiptext This is the PopUpButton Accessibility Class.
 *  @review
 */
public class PopUpButtonAccImpl extends AccImpl
{
    include "../core/Version.as";

	//--------------------------------------------------------------------------
	//
	//  Class initialization
	//
	//--------------------------------------------------------------------------

	/**
	 *  Static variable triggering the <code>hookAccessibility()</code> method.
	 *  This is used for initializing <code>PopUpButtonAccImpl</code> class to hook its
	 *  <code>createAccessibilityImplementation()</code> method to the <code>PopUpButton</code> class 
	 *  before it gets called when <code>UIComponent</code> invokes the <code>initializeAccessibility()</code> method.
	 * 
	 *  @see mx.accessibility.AccImpl#createAccessibilityImplementation()
	 *  @see mx.core.UIComponent#createAccessibilityImplementation()
	 *  @see mx.core.UIComponent#initializeAccessibility()
	 *  @see mx.core.UIComponent#initialize()
	 */
	private static var accessibilityHooked:Boolean = hookAccessibility();

	/**
	 *  Static method that swaps the <code>createAccessibilityImplementation()</code>
	 *  method of <code>UIComponent</code> subclass with the appropriate <code>AccImpl</code> subclass.
	 * 
	 *  @see mx.accessibility.AccImpl#createAccessibilityImplementation()
	 *  @see mx.core.UIComponent#createAccessibilityImplementation()
	 *  @see mx.core.UIComponent#initializeAccessibility()
	 *  @see mx.core.UIComponent#initialize()
	 */
	private static function hookAccessibility():Boolean
	{
		PopUpButton.createAccessibilityImplementation =
			createAccessibilityImplementation;
		return true;
	}

	//--------------------------------------------------------------------------
	//
	//  Class constants
	//
	//--------------------------------------------------------------------------

	/**
	 *  The component is pressed. 
	 */
	private static const STATE_SYSTEM_PRESSED:uint = 0x00000008;
	
	/**
	 *  The component is pressed. 
	 */
	private static const STATE_SYSTEM_HOTTRACKED:uint = 0x00000080;
	
	/**
      *  Object displays a pop-up menu or window when invoked. 
      */
     private static const STATE_SYSTEM_HASPOPUP:uint = 0x40000000;

	/**
	 *  An object's name property has changed. 
	 */
	private static const EVENT_OBJECT_NAMECHANGE:uint = 0x800C;
	
	/**
	 *  An object's state has changed.
	 */
	private static const EVENT_OBJECT_STATECHANGE:uint = 0x800A;
	
	/**
	 *  The role represents a button on a toolbar that has a drop-down list icon directly adjacent to the button. 
	 */
	private static const ROLE_SYSTEM_SPLITBUTTON:uint = 0x3e;
	
	/**
	 *  The object represents a button that drops down a list of items. 
	 */
	private static const ROLE_SYSTEM_BUTTONDROPDOWN:uint = 0x38;
	
	
	//--------------------------------------------------------------------------
	//
	//  Class methods
	//
	//--------------------------------------------------------------------------

	/**
	 *  Method for creating the Accessibility Implementation class for a component.
	 *  <p>This method is called by the <code>initializeAccessibility()</code> method for the <code>UIComponent</code> subclass when the component initalizes.</p>
	 *  <p>All <code>AccImpl</code> subclasses must implement this method</p>
	 * 
	 * @example Following is an example showing how the <code>createAccessibilityImplementation()</code> method is implemented in <code>PopUpButtonAccImpl</code>, 
	 * the accessibility implementation for the <code>mx.controls.PopUpButton</code> component.
	 * <listing version="3.0">mx_internal static function createAccessibilityImplementation(
	 * 							component:UIComponent):void
	 * {
	 * 	component.accessibilityImplementation =
	 * 		new PopUpButtonAccImpl(component);
	 * }</listing>
	 * 
	 *  @param component The UIComponent instance that this PopUpButtonAccImpl instance makes accessible.
	 * 
	 *  @see mx.accessibility.AccImpl#createAccessibilityImplementation()
	 *  @see mx.core.UIComponent#createAccessibilityImplementation() 
	 *  @see mx.core.UIComponent#initalizeAccessibility()
	 *  @see mx.core.UIComponent#initialize()
	 */
	mx_internal static function createAccessibilityImplementation(
								component:UIComponent):void
	{
		component.accessibilityImplementation =
			new PopUpButtonAccImpl(component);
	}

	/**
	 *  Method call for enabling accessibility for a component.
	 *  This method is required for the compiler to activate
	 *  the accessibility classes for a component.
	 * 
	 *  <p>When a Flex project is compiled with the Generate accessible SWF file set to <code>true</code>, the Flex compiler 
	 *  automatically enables accessibility implementations for the components used in the project that have them.</p>
	 * 
	 *  <p>Accessible component classes should include the <code>[AccessibilityClass]</code>
	 *  meta tag to specify the appropriate <code>AccImpl</code> subclass or accessibility implementation and let the compiler know to call 
	 *  <code>enableAccessibility()</code>.</p>
	 * 
	 *  @example Following is an example showing how the <code>[AccessibilityClass]</code> meta tag specifies the appropriate <code>AccImpl</code> 
	 * subclass or accessibility implementation for the <code>mx.controls.PopUpButton</code> component.
	 * <listing version="3.0">[AccessibilityClass(implementation="mx.accessibility.PopUpButtonAccImpl")]</listing>
	 * 
	 *  @see ../../../html/help.html?content=accessible_4.html Configuring Flex applications for accessibility
	 */
	public static function enableAccessibility():void
	{
	}

	//--------------------------------------------------------------------------
	//
	//  Constructor
	//
	//--------------------------------------------------------------------------

	/**
	 * Creates a new PopUpButtonAccImpl instance for the specified PopUpButton component.
	 *
	 * <p>Direct calls to the AccImpl subclass constructors are unneccessary.
	 * When a Flex project is compiled with Generate accessible SWF file set to <code>true</code>, 
	 * the compiler instantiates the accessibility implementations for the components used in the 
	 * project that have them by calling the the static <code>enableAccessibility()</code> method.</p>
	 *
	 * @param master The UIComponent instance that this PopUpButtonAccImpl instance is making accessible.
	 * 
	 * @see ../../../html/help.html?content=accessible_4.html Configuring Flex applications for accessibility
	 */
	public function PopUpButtonAccImpl(master:UIComponent)
	{
		super(master);

		role = ROLE_SYSTEM_SPLITBUTTON;
	}

	//--------------------------------------------------------------------------
	//
	//  Overridden properties: AccImpl
	//
	//--------------------------------------------------------------------------

	//----------------------------------
	//  eventsToHandle
	//----------------------------------

	/**
	 *	@inheritDoc
	 */
	override protected function get eventsToHandle():Array
	{
		return super.eventsToHandle.concat([ "click", "labelChanged" ]);
	}
	
	//--------------------------------------------------------------------------
	//
	//  Overridden methods: AccessibilityImplementation
	//
	//--------------------------------------------------------------------------
	/**
	 * IAccessible method for returning system role for the component.
	 * System roles are predefined for all the components in MSAA.
	 * <p>A PopUpButton component, ( &lt;code&gt;childID == 0&lt;/code&gt; ), reports the role &lt;code&gt;ROLE_SYSTEM_SPLITBUTTON&lt;/code&gt; (0x3E).
	 * The PopUpButton's drop down arrow ( &lt;code&gt;childID == 1&lt;/code&gt; ) reports the role &lt;code&gt;ROLE_SYSTEM_BUTTONDROPDOWN&lt;/code&gt; (0x3B).</p>
	 *
	 * @param childID An unsigned integer corresponding to one of the component's child elements as defined by 
	 * <code><a href="#getChildIDArray()">getChildIDArray()</a></code>.
	 *
	 * @return Role associated with the component. 
	 *  
	 * @tiptext Returns the system role for the component
	 * @helpid 3009
	 * 
	 * @see ../../flash/accessibility/constants.html#roles AccessibilityImplementation Constants: Object Roles
	 * @see http://msdn.microsoft.com/en-us/library/ms696113(VS.85).aspx Microsoft Accessibility Developer Center: IAccessible::get_accRole
	 */

	override public function get_accRole(childID:uint):uint
	{
		if (childID == 0)
			return role;
			
		return ROLE_SYSTEM_BUTTONDROPDOWN;
	}
	
	/**
	 *  IAccessible method for returning the value of the PopUpButton
	 *  which is spoken out by the screen reader.
	 *  The PopUpButton should return value that would be applied when clicked. For example, the value of the PopUpButton's label property.
	 *
	 * @param childID An unsigned integer corresponding to one of the component's child elements as defined by 
	 * <code><a href="#getChildIDArray()">getChildIDArray()</a></code>.
	 *
	 *  @return Value String
	 * 
	 *  @see http://msdn.microsoft.com/en-us/library/ms697312(VS.85).aspx Microsoft Accessibility Developer Center: IAccessible::get_accValue
	 *  @see http://msdn.microsoft.com/en-us/library/bb404170.aspx#ActiveAccessibility2007OfficeFluentUI_TheSplitButtonControl Working with Active Accessibility in the 2007 Office Fluent User Interface: The SplitButton Control
	 */

	override public function get_accValue(childID:uint):String
	{
		var accValue:String;
		
		// local reference to the master component as a PopUpButton
		var popUpButton:PopUpButton = PopUpButton(master);
		
		if(childID == 0){
		// the PopUpButton component itself
			
			// local reference to the label
			var label:String = popUpButton.label;
			
			if(popUpButton.popUp 
				&& popUpButton.popUp is Menu
				&& label != null
				&& label != ""){
				// If the popUp exists and is a Menu and 
				// the label exists and is not an empty string...
				var popUpMenu:Menu = popUpButton.popUp as Menu;
				
				if(popUpMenu.itemToLabel(popUpMenu.selectedItem) == label){
					// If the label matches the popUp menu's selectedIndex, return the label as the accValue.
					accValue = popUpMenu.itemToLabel(popUpMenu.selectedItem);
					if(popUpButton.accessibilityProperties && popUpButton.accessibilityProperties.shortcut){
						// If a keyboard shortcut is defined in the 
						// PopUpButton's .accessibilityProperties object,
						// append it to the returned accValue.
						accValue += " ("+popUpButton.accessibilityProperties.shortcut+")";
					}
				}
				
			}
		}

		return accValue;
	}

	
	/**
	 *  IAccessible method for returning the state of the PopUpButton.
	 *  States are predefined for all the components in MSAA.
	 *  Values are assigned to each state.
	 *  Depending upon the PopUpButton being pressed or released, its popUp being present and opened or closed, and the PopUpButton being hovered over or focused, 
	 *  a value is returned.
	 * 
	 * @param	childID An unsigned integer corresponding to one of the component's child elements as defined by 
	 * <code><a href="#getChildIDArray()">getChildIDArray()</a></code>.
	 * 
	 * @return State indicating whether the PopUpButton is pressed or released.
	 * 
	 * @see #getChildIDArray()
	 * @see flash.accessibility.AccessibilityImplementation#get_accState()
	 * @see ../../flash/accessibility/constants.html#states AccessibilityImplementation Constants: Object State Constants
	 * @see http://msdn.microsoft.com/en-us/library/ms696191(VS.85).aspx Microsoft Accessibility Developer Center: IAccessible::get_accState
	 */
	override public function get_accState(childID:uint):uint
	{
		// the normal default state
		var accState:uint = 0;
		
		// local reference to the master component as a PopUpButton
		var popUpButton:PopUpButton = PopUpButton(master);
		
		
		if(childID == 1){
		// the drop-down button 	
			
			if(popUpButton.popUp){
			// if the PopUpButton's popUp property is defined, 
			// indicate that the drop-down button has a pop up.	
				accState = STATE_SYSTEM_HASPOPUP;
			}
			
			
			if(popUpButton.mx_internal::isShowingPopUp == true){
			// if the popUp is showing, 
			// indicate that the drop-down button is pressed
				accState |= STATE_SYSTEM_PRESSED; 
			}
			
		} else {
		// the PopUpButton component itself
		 	
		 	// the component state inherited from AccImpl
		 	// unavailable, normal, focusable, or focused
		 	accState = getState(childID);
		 	
		 	
		 	if (popUpButton.selected) {
		 	// if the popUpButton is selected,
		 	// indicate that popUpButton is pressed
		 		accState |= STATE_SYSTEM_PRESSED;
		 	}
		 	
		}
		
		var mouseX:Number = master.mouseX;
		var mouseY:Number = master.mouseY;
		var bounds:Rectangle = master.getBounds(master);
		if((mouseX >= bounds.x 
			&& mouseX <= (bounds.x + bounds.width)
			&& mouseY >= bounds.y
			&& mouseY <= (bounds.y + bounds.height))
			|| (popUpButton.focusManager.getFocus() == popUpButton)){
			// if the parent popUpButton component or child drop-down button 
			// has either mouse or keyboard focus, 
			// indicate that it is hot-tracked.
			accState |= STATE_SYSTEM_HOTTRACKED;
		}

		return accState;
	}

	
	/**
	 *  Method to return an array of childIDs.
	 *
	 *  @return An array of unsigned integer IDs with a length of one for the drop-down menu button. 
	 */
	override public function getChildIDArray():Array
	{
		var childIDs:Array = [];
		
		
		for (var i:int = 0; i < 1; i++)
		{
			childIDs[i] = i + 1;
		}
		
		return childIDs;
	}

	/**
	 *  IAccessible method for returning the default action of the PopUpButton, which is Press for the component itself and Open for its child drop-down menu button.
	 *
	 *  @param childID An unsigned integer corresponding to one of the component's child elements as defined by 
	 *  <code><a href="#getChildIDArray()">getChildIDArray()</a></code>.
	 *
	 *  @return DefaultAction String
	 * 
	 *  @see http://msdn.microsoft.com/en-us/library/ms696144(VS.85).aspx Microsoft Accessibility Developer Center: IAccessible::get_accDefaultAction
	 */
	override public function get_accDefaultAction(childID:uint):String
	{
		if(childID == 0){
			return "Press";
		}
		return "Open";
	}


	/**
	 *  IAccessible method for performing the default action of the PopUpButton, which is Press for the component itself and Open for its child drop-down menu button.
	 *
	 *  @param childID An unsigned integer corresponding to one of the component's child elements as defined by 
     *  <code><a href="#getChildIDArray()">getChildIDArray()</a></code>.
	 * 
	 *  @see http://msdn.microsoft.com/en-us/library/ms696119(VS.85).aspx Microsoft Accessibility Developer Center: IAccessible::accDoDefaultAction
	 */

	override public function accDoDefaultAction(childID:uint):void
	{
		var popUpButton:mx.controls.PopUpButton = master as mx.controls.PopUpButton;
		if (childID==0)
		{
			var event:KeyboardEvent = new KeyboardEvent(KeyboardEvent.KEY_DOWN);
			event.keyCode = Keyboard.SPACE;
			master.dispatchEvent(event);

			event = new KeyboardEvent(KeyboardEvent.KEY_UP);
			event.keyCode = Keyboard.SPACE;
			master.dispatchEvent(event);
			
		} else if(childID == 1){
			
			if(popUpButton.mx_internal::isShowingPopUp == true){
				popUpButton.close();
			} else {
				popUpButton.open();
			}
		}
	}
	
	//--------------------------------------------------------------------------
	//
	//  Overridden methods: AccImpl
	//
	//--------------------------------------------------------------------------

	/**
	 *  Method for returning the name of the PopUpButton which is spoken out by the screen reader.
	 *  The PopUpButton should return the label inside as the name of the PopUpButton. 
	 *  The child drop-down button should retrurn "Open" or "Close".
	 *
	 *  @param	childID An unsigned integer corresponding to one of the component's child elements as defined by 
	 *  <code><a href="#getChildIDArray()">getChildIDArray()</a></code>.
	 *
	 *  @return Name string corresponding to the PopUpButton label.  "Open" or "Close" for the child drop-down button. 
	 */

	override protected function getName(childID:uint):String
	{	
		var popUpButton:mx.controls.PopUpButton = master as mx.controls.PopUpButton;
		var popUp:UIComponent = popUpButton.popUp as UIComponent;
		if(popUp){
			if(!popUp.accessibilityProperties){
				popUp.accessibilityProperties = new AccessibilityProperties();
			}
			if(popUp.accessibilityProperties.name == ""){
				popUp.accessibilityProperties.name = "Context";
			}
		}
		if(childID == 1){
			return (popUpButton.mx_internal::isShowingPopUp == true) ? "Close" : "Open";
		}
		var label:String = popUpButton.label;
		return label != null && label != "" ? label + " PopUpButton" : "PopUpButton";
	}
	
	/**
	 *  IAccessible method for returning the bounding box of a the PopUpButton or its child drop-down arrow.
	 * 
	 *  @param childID An unsigned integer corresponding to one of the component's child elements as defined by 
	 *  <code><a href="#getChildIDArray()">getChildIDArray()</a></code>.
	 *
	 *  @return The PopUpButton or its drop-down button.
	 * 
	 *  @see http://msdn.microsoft.com/en-us/library/ms696118(VS.85).aspx Microsoft Accessibility Developer Center: IAccessible::accLocation
	 */
	override public function accLocation(childID:uint):*
	{
		// the master component
		var location:* = master;
		
		if(childID == 1){
		// calculate the rectangle location of the child drop-down button
			var popUpButton:PopUpButton = master as PopUpButton;
			var popUpButtonRect:Rectangle = popUpButton.getRect(popUpButton);
			var arrowButtonsWidth:Number = popUpButton.mx_internal::getArrowButtonsWidth();
			location = new Rectangle(
								popUpButtonRect.x + popUpButton.width - arrowButtonsWidth,
								popUpButtonRect.y,
								arrowButtonsWidth,
								popUpButton.height
							);	
		}
		
		return location;
		
	}


	//--------------------------------------------------------------------------
	//
	//  Overridden event handlers: AccImpl
	//
	//--------------------------------------------------------------------------

	/**
	 *  Override the generic event handler.
	 *  Each AccImpl subclass must implement this method to listen for events from its master component. 
	 * 
	 *  @param event The event object.
	 * 
	 *  @see mx.accessibility.AccImpl#eventHandler()
	 */
	override protected function eventHandler(event:Event):void
	{
		switch (event.type)
		{
			case "click":
			{
				Accessibility.sendEvent(master, 0, EVENT_OBJECT_STATECHANGE);
				Accessibility.updateProperties();
				break;
			}
			
			case "labelChanged":
			{
				Accessibility.sendEvent(master, 0, EVENT_OBJECT_NAMECHANGE);
				Accessibility.updateProperties();
				break;
			}
		}
	}
}

}
